function [dx, duals, Gmtrx, Tmtrxf, Tmtrxt, bind_Af, bind_At, bind_vu, bind_vl] = LPfslp_d			...
				(alg,										...
				baseMVA, gen, branch, bus, ref, pv, pq, nPPg,					...
				f_lp0, beta, Mppg, Emtrx, PPg, PPg_u, stepsize_P, stepsize_Q, norm_slackvar,	...
				dSbus_dVm, dSbus_dVa,								...
				dAf_dVa, dAf_dVm, dAt_dVa, dAt_dVm)

%Once the solution of a load flow is given, LPfslp_d formulates a LP problem, and invokes a LP solver 
%to solve the problem. The formulation is dense.

%   MATPOWER Version 1.0.1
%   by Deqiang (David) Gan, PSERC Cornell    9/19/97
%   Copyright (c) 1997 by Power System Engineering Research Center (PSERC)
%   See http://www.pserc.cornell.edu/ for more info.

% ------ get ready to go ------

%% define named indices into bus, gen, branch matrices

[PQ, PV, REF, NONE, BUS_I, BUS_TYPE, PD, QD, GS, BS, BUS_AREA, VM, ...
	VA, BASE_KV, ZONE, VMAX, VMIN, LAM_P, LAM_Q, MU_VMAX, MU_VMIN] = idx_bus;
[GEN_BUS, PG, QG, QMAX, QMIN, VG, MBASE, ...
	GEN_STATUS, PMAX, PMIN, MU_PMAX, MU_PMIN, MU_QMAX, MU_QMIN] = idx_gen;
[F_BUS, T_BUS, BR_R, BR_X, BR_B, RATE_A, RATE_B, ...
	RATE_C, TAP, SHIFT, BR_STATUS, PF, QF, PT, QT, MU_SF, MU_ST] = idx_brch;

%% set up constants and intial values (in quasi-alphabetical order)

on = find(gen(:, GEN_STATUS));								%% which gens are on-line
gbus = gen(on, GEN_BUS);									%% what buses are they at?	
j = sqrt (-1);												%% operator of complex numbers
nb = size(bus, 1);											%% number of buses
nb1= nb-1;													%% frequently used number
nl = size(branch, 1);										%% number of branches
npq	= length(pq);											%% number of pq buses
npv	= length(pv);											%% number of pv buses
ng = npv +1;												%% number of on-line gens
pvpq = [1:ref-1, ref+1:size(bus, 1)];						%% indices of pv & pq buses

idx_Pg  = 1			: ng;
idx_Qg  = ng+1		: ng+ng;
idx_PPg = ng+ng+1	: ng+ng+nPPg;


% ----- get constraints transformation matrix -----

jacobian 	=[	real(dSbus_dVm(pvpq, :)),	real(dSbus_dVa(pvpq, pvpq))
	 			imag(dSbus_dVm),			imag(dSbus_dVa(:, pvpq))];

Gmtrx = jacobian \ [Emtrx(pvpq,:), sparse(nb1, ng); sparse(nb, ng), Emtrx]; Gmtrx = full(Gmtrx);

%	[LL, UU] = lu(jacobian);
%	yy = LL \ [Emtrx(pvpq,:), sparse(nb1, ng); sparse(nb, ng), Emtrx];
%	Gmtrx = UU \ yy;

Tmtrxf = [dAf_dVm, dAf_dVa(:, pvpq)] * Gmtrx;
Tmtrxt = [dAt_dVm, dAt_dVa(:, pvpq)] * Gmtrx;


%% ----- formulate and solve LP using constraints relaxation approach -----

jg= [ real(dSbus_dVm(ref,:)), real(dSbus_dVa(ref, pvpq))] * Gmtrx;		%% load balance equation
Af = (abs(branch(:, PF)+ j*branch(:, QF)))/baseMVA;
At = (abs(branch(:, PT)+ j*branch(:, QT)))/baseMVA;

% identify initial binding sets
bind_Af = find( (branch(:, RATE_A))/baseMVA - Af < 0.15* (branch(:, RATE_A))/baseMVA);
bind_At = find( (branch(:, RATE_A))/baseMVA - At < 0.15* (branch(:, RATE_A))/baseMVA);
bind_vu = find( bus(:, VMAX) - bus(:, VM) 	< 0.015 );
bind_vl = find( bus(:, VM)   - bus(:, VMIN) < 0.015 );


lp_conv = 0;	ilp = 0;

while lp_conv == 0

	ilp = ilp+1;



	%% formulate LP

	a_lp = [
		speye(ng),					sparse(ng, ng),				-Mppg	%% translation equations
		jg(idx_Pg)-Emtrx(ref, :),	jg(idx_Qg),					sparse(1, nPPg)					%% load balance equation
		Tmtrxf(bind_Af,	idx_Pg),	Tmtrxf(bind_Af, idx_Qg),	sparse(length(bind_Af), nPPg)	%% thermal limits
		Tmtrxt(bind_At, idx_Pg),	Tmtrxt(bind_At, idx_Qg),	sparse(length(bind_At), nPPg)	%% thermal limits
		Gmtrx(bind_vu, 	idx_Pg),	Gmtrx(bind_vu, 	idx_Qg),	sparse(length(bind_vu), nPPg)	%% voltage limits
		-Gmtrx(bind_vl, idx_Pg),	-Gmtrx(bind_vl, idx_Qg),	sparse(length(bind_vl), nPPg)	%% voltage limits
		speye(ng),					sparse(ng, ng),				sparse(ng, nPPg)				%% Pg upper limits
		-speye(ng),					sparse(ng, ng),				sparse(ng, nPPg)				%% Pg lower limits
		sparse(ng, ng),				speye(ng),					sparse(ng, nPPg)				%% Qg upper limits
		sparse(ng, ng),				-speye(ng),					sparse(ng, nPPg)				%% Qg lower limits
			];

	rhs_lp=[
		zeros(ng, 1)												%% translation equations
		0															%% load balance equation
		branch(bind_Af, RATE_A)/baseMVA - Af(bind_Af, 1)			%% thermal limits
		branch(bind_At, RATE_A)/baseMVA - At(bind_At, 1)			%% thermal limits
		bus(bind_vu, VMAX) 				- bus(bind_vu, VM)			%% voltage limits
		bus(bind_vl, VM)				- bus(bind_vl, VMIN)		%% voltage limits
		gen(on, PMAX)/baseMVA			- gen(on, PG)/baseMVA		%% Pg upper bounds
		gen(on, PG)/baseMVA				- gen(on, PMIN)/baseMVA		%% Pg lower bounds
		gen(on, QMAX)/baseMVA 			- gen(on, QG)/baseMVA		%% Qg upper bounds
		gen(on, QG)/baseMVA			 	- gen(on, QMIN)/baseMVA		%% Qg lower bounds
	 		];	
	
	f_lp = f_lp0;
	vub = [ stepsize_P*ones(ng, 1);  stepsize_Q*ones(ng, 1); PPg_u - PPg];
	vlb = [-stepsize_P*ones(ng, 1); -stepsize_Q*ones(ng, 1); -PPg];
	constr_sense = []; 
	constr_sense (1:ng+1) = zeros(ng+1, 1); 
	constr_sense(ng+1+1:length(rhs_lp))=-1*ones(length(rhs_lp)-1-ng, 1);
	num_of_equations = ng+1;
	
		
	
	%% add slack variables to ensure the solution of LP exists
	
	if norm_slackvar > 1.0e-12
		n_slack = length(rhs_lp) - ng;
		temp = [sparse(ng, n_slack);			%% the PPg-Pg translation equations are not slackable
			-1*speye(n_slack)];
		a_lp = [a_lp, temp];
		vub = [vub; 1.0e2*ones(n_slack, 1)];
		vlb = [vlb; zeros(n_slack, 1)];
		f_lp = [f_lp0;	80*baseMVA*max(beta) *ones(n_slack, 1)];
	end;


	[dx, duals] = LPlpslvr(alg, a_lp, f_lp, rhs_lp, constr_sense, num_of_equations, vlb, vub);	


	%% check convergence of inner iteration
	% the criteria is: inner iteration converges if there is no NEW violations

	dAf = Tmtrxf * dx(1:ng+ng);	
	dAt = Tmtrxt * dx(1:ng+ng);
	dVm = Gmtrx(1:nb, :) * dx(1:ng+ng);
	Af_violation = find((branch(:, RATE_A))/baseMVA - Af - dAf  < -1.0e-4 );
	At_violation = find((branch(:, RATE_A))/baseMVA - At - dAt  < -1.0e-4 );
	vu_violation = find( bus(:, VMAX) - bus(:, VM)   - dVm < -1.0e-5);
	vl_violation = find( bus(:, VM)   - bus(:, VMIN) + dVm < -1.0e-5);
	
	temp1=zeros(nl, 1); temp1(bind_Af)=ones(length(bind_Af), 1);
	temp2=temp1;		temp2(Af_violation)=ones(length(Af_violation), 1);
	new_Af_violation=find(temp2-temp1);
	bind_Af = find(temp2);
			
	temp1=zeros(nl, 1); temp1(bind_At)=ones(length(bind_At), 1);
	temp2=temp1;		temp2(At_violation)=ones(length(At_violation), 1);
	new_At_violation=find(temp2-temp1);
	bind_At = find(temp2);
	
	temp1=zeros(nb, 1); temp1(bind_vu)=ones(length(bind_vu), 1);
	temp2=temp1;		temp2(vu_violation)=ones(length(vu_violation), 1);
	new_vu_violation=find(temp2-temp1);
	bind_vu = find(temp2);
	
	temp1=zeros(nb, 1); temp1(bind_vl)=ones(length(bind_vl), 1);
	temp2=temp1;		temp2(vl_violation)=ones(length(vl_violation), 1);
	new_vl_violation=find(temp2-temp1);
	bind_vl = find(temp2);
	
	numofviolation = sum(length(new_Af_violation)+length(new_At_violation)+length(new_vu_violation)+length(new_vl_violation));
	% fprintf('\n 		number of violations in lp = %i \', numofviolation);
	if  numofviolation == 0, lp_conv = 1;	end;
	
end

return;

